#include "hardware.h"
#include "vr_library.h"

// Initialise the hardware
void appInitHardware(void) {
	initHardware();
}
// Initialise the software
TICK_COUNT appInitSoftware(TICK_COUNT loopStart){
	return 0;
}
// This is the main loop
TICK_COUNT appControl(LOOP_COUNT loopCount, TICK_COUNT loopStart) {

	// -------- Start Switch/Button-------
	// Switch/Button - see switch.h
	
	// To test if it is pressed then
	if(SWITCH_pressed(&button)){
		// pressed
	}
	
	// To test if it is released then
	if(SWITCH_released(&button)){
		// released
	}
	// -------- End   Switch/Button-------

	// -------- Start Marquee-------
	// Marquee - see 'segled.h'
	// Before using the Marquee you need to redirect rprintf to write to it
	// This can be done using
	Writer old = rprintfInit(marqueeGetWriter(&marquee));
	
	// All rprintf output will then be sent to the marquee but will not
	// display until an end-of-line eg "\n" has been sent. Example:-
	// rprintf("Hello World\n");
	
	// If the endDelay is non-zero then the marquee will scroll
	// forever or until you call: marqueeStop(&marquee);
	
	// If the endDelay is zero then the marquee will stop once
	// the entire line has been shown ('one-shot' mode)
	
	// In 'one-shot' mode then you may want to make sure that
	// a previous line has finished before you display a second line.
	// This can be done as follows:-
	marqueeSetEndDelay(&marquee,0); // Make sure we are in one-shot mode
	if(marqueeIsActive(&marquee)==FALSE){
	     if(loopCount==1){
	     	rprintf("ABCDEFGHIJKLMNOPQRSTUVWXYZ\n");
	     }else{
			rprintf("Loop=%u\n",(unsigned)loopCount); // Put the loop count
	     }
	}
	
	// Restore rprintf back to its previous location
	rprintfInit(old);
	// -------- End   Marquee-------

	// -------- Start Text to Speech-------
	// Say an English phrase
	sayText("Death to all humans");
	// -------- End   Text to Speech-------

	// -------- Start PING sonar-------
	// Read the PING sonar and store the result in sonar.distance.cm
	distanceRead(sonar);
	
	// The value can be printed using %u eg rprintf("Distance=%u",sonar.distance.cm);
	// or dumped using:
	rprintf("sonar: ");
	distanceDump(sonar);
	rprintfCRLF();
	// -------- End   PING sonar-------

	// -------- Start LED-------
	// The LED can be manipulated using the calls in led.h
	
	// To turn the LED on:-
	LED_on(&led);
	
	// To turn the LED off:-
	LED_off(&led);
	// -------- End   LED-------

	// -------- Start Actuators -------
	// To control your.motors/servos then see actuators.h in the manual
	// To retrieve the required speed of motor_left use:
	// DRIVE_SPEED speed=act_getSpeed(motor_left);
	// To set the required speed of motor_left use:
	// act_setSpeed(motor_left,speed);
	// This example will move the motors back and forth using the loopStart time:
	TICK_COUNT ms = loopStart / 1000;		// Get current time in ms
	int16_t now = ms % (TICK_COUNT)10000; 	// 10 sec for a full swing
	if(now >= (int16_t)5000){				// Goes from 0ms...5000ms
		now = (int16_t)10000 - now;			// then 5000ms...0ms
	}
	// Map it into DRIVE_SPEED range
	DRIVE_SPEED speed = interpolate(now, 0, 5000, DRIVE_SPEED_MIN, DRIVE_SPEED_MAX);
	// Set speed for all motors/servos
	act_setSpeed(&motor_left,speed);
	act_setSpeed(&motor_right,speed);
	// -------- End   Actuators -------

	return 0;
}
