//VR Recognition Module for Axon

#define DEFAULT_LANGUAGE 'A'//set to English

uint8_t VR_group=1;//default Group


MAKE_WRITER(writerVoice_Recognition)
	{
	return Voice_RecognitionSendByte(byte);
	}


//set language for SI recognition
//Language (A = English, B = Italian, C = Japanese, D = German) 
void VR_Module_change_language(char lang)
	{
	//transmit data to VR module, store previous UART setting
	Writer old = rprintfInit(&writerVoice_Recognition);

	rprintf("l");//change language
	rprintf("%c",lang);

	while(Voice_RecognitionGetByte() != 'o')//wait until language changed
		delay_ms(20);
	
	rprintfInit(old);//use default output method
	}

//initialization, put in software init
void VR_Module_init(void)
	{
	bufferFlush(&rx_Voice_Recognition_Buffer);
	bufferFlush(&tx_Voice_Recognition_Buffer);

	rprintf("\nvoice recognition initializing ...");

	delay_ms(1000);//required boot up delay

	//transmit data to VR module, store previous UART setting
	Writer old = rprintfInit(&writerVoice_Recognition);


	//wake up
	while(Voice_RecognitionGetByte() != 'o')
		{
		rprintf("b");
		delay_ms(300);
		}

	rprintfInit(old);//use default output method
	rprintf(" initialized");
	//transmit data to VR module, store previous UART setting
	rprintfInit(&writerVoice_Recognition);


	//ask for firmware id (not required)
	//rprintf("x");
	//while(Voice_RecognitionGetByte() != 'x');

	//set language
	VR_Module_change_language(DEFAULT_LANGUAGE);

	rprintfInit(old);//use default output method
	rprintf("\nDefault language changed.");
	//transmit data to VR module, store previous UART setting
	rprintfInit(&writerVoice_Recognition);

	//set timeout to 5 seconds
	rprintf("o");
	rprintf("F");
	while(Voice_RecognitionGetByte() != 'o')//wait until changed
		delay_ms(20);

	rprintfInit(old);//use default output method
	rprintf("\nTiming set.");
	//transmit data to VR module, store previous UART setting
	rprintfInit(&writerVoice_Recognition);

	//change knob
	rprintf("k");
	rprintf("B");//A to E, E being fewer valid results
	while(Voice_RecognitionGetByte() != 'o')//wait until changed
		delay_ms(20);
	//change level
	rprintf("v");
	rprintf("B");//B to F, F gives least false positives
	while(Voice_RecognitionGetByte() != 'o')//wait until changed
		delay_ms(20);

	rprintfInit(old);//use default output method
	rprintf("\nSensitivity set.");
	//transmit data to VR module, store previous UART setting
	rprintfInit(&writerVoice_Recognition);

	rprintfInit(old);//use default output method
	
	rprintf("\n.....initialized.");
	}


//start voice recognition (0-3)
void VR_Module_start_recognition(uint8_t group)
	{
	char number='B';

	rprintf("\nvoice recognition activating...");

	//transmit data to VR module, store previous UART setting
	Writer old = rprintfInit(&writerVoice_Recognition);

	if(group==0)//trigger
		number='A';
	else if(group==1)//1-15, generic
		number='B';
	else if(group==2)
		number='C';
	else if(group==3)
		number='D';
	else if(group==16)//16, password
		number='Q';
		

	rprintf("d");
	rprintf("%c",number);

	rprintfInit(old);//use default output method

	rprintf("\n..... group %d activated.",group);
	}


//gets result of voice recognition
// -1 = timed out, no word detected
// -2 = no word detected yet within time limit
// any other value represents detect word
//note: may take several seconds before result is returned
int8_t VR_Module_get_result(void)
	{
	char data = Voice_RecognitionGetByte();

	//no data received
	if ((uint8_t)data == -1)
		return -2;

	//if positive result
	else if (data == 'r')
		{
		rprintf("\ngot positive result");

		//transmit data to VR module, store previous UART setting
		Writer old = rprintfInit(&writerVoice_Recognition);
		rprintf(" ");
		rprintfInit(old);//use default output method

		//wait for returned result
		data = Voice_RecognitionGetByte();
		while((int8_t)data == -1)
			data = Voice_RecognitionGetByte();
		
		return data - 'A';
		}

	//timed out
	else if (data == 't')
		{
		rprintf("\nvoice detection timed out");
		return -1;
		}

	else if (data == 'e')
		{
		char error_code=0;

		data=-1;//reset value

		Writer old = rprintfInit(&writerVoice_Recognition);
		
		//get error code
		rprintf(" ");
		while((int8_t)data == -1)
			data = Voice_RecognitionGetByte();
		error_code = (data - 'A') * 16;
		rprintf(" ");
		while((int8_t)data == -1)
			data = Voice_RecognitionGetByte();
		error_code = error_code + (data - 'A');


		rprintfInit(old);//use default output method
		
		//note that manual actually specifies error codes
		rprintf("\nvoice error detected: %c", error_code);
		return -1;
		}
	return -2;
	}
